package com.unistrong.pogoservicelib.pogoservice.serial.template;

import android.util.Log;

import com.aill.androidserialport.SerialPort;
import com.unistrong.pogoservicelib.BuildConfig;
import com.unistrong.pogoservicelib.pogoservice.status.SerialStatus;
import com.unistrong.pogoservicelib.pogoservice.status.TranslateStatus;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import static com.unistrong.pogoservicelib.pogoservice.status.SerialStatus.CONNECTED;
import static com.unistrong.pogoservicelib.pogoservice.status.SerialStatus.DISCONNECTED;
import static com.unistrong.pogoservicelib.pogoservice.status.TranslateStatus.FAILED;
import static com.unistrong.pogoservicelib.pogoservice.status.TranslateStatus.IDLE;
import static com.unistrong.pogoservicelib.pogoservice.status.TranslateStatus.SUCCEED;

public class SerialPortImpl implements ISerialPort {
    private static String TAG = "SerialPortImpl";
    private static boolean DEBUG = BuildConfig.DEBUG;

    protected String mPath = null;
    private SerialPort mSerialPort;
    private FileInputStream mInput;
    private FileOutputStream mOutput;
    protected IDataReceiveCallback mDataRecvCallback;
    private SerialStatus mConnectStatus = DISCONNECTED;
    private TranslateStatus mTranslateStatus = IDLE;

    /**
     * register serial port data receiver
     * @param onReceiveCallback Data received callback
     */
    public void registerReceiveCallback(IDataReceiveCallback onReceiveCallback) {
        mDataRecvCallback = onReceiveCallback;
    }

    public void unregisterReceiveCallback() {
        mDataRecvCallback = null;
    }

    /**
     * Open serial port
     * @param baudRate baud rate, such as 9600
     * @return open serial success or not, true for success.
     */
    @Override
    public boolean open(int baudRate) {
        if (null != mSerialPort) mSerialPort.close();
        if (null == mPath) return false;
        try {
            mSerialPort = new SerialPort(new File(mPath), baudRate, 0);
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        mInput = (FileInputStream) mSerialPort.getInputStream();
        mOutput = (FileOutputStream) mSerialPort.getOutputStream();
        return true;
    }

    /**
     * Close serial port
     */
    @Override
    public void close() {
        try {
            if (null != mInput) {
                mInput.close();
            }
            if (null != mOutput) {
                mOutput.close();
            }
            if (null != mSerialPort) {
                mSerialPort.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            mInput = null;
            mOutput = null;
            mSerialPort = null;
        }
    }

    /**
     * Send data to serial port
     * @param data data to be send.
     * @param len data length
     */
    @Override
    public void sendData(byte[] data, int len) {
        try {
            mOutput.write(data);
            mOutput.flush();
            mTranslateStatus = SUCCEED;
        } catch (Exception e) {
            Log.e(TAG, "Send data failed! " + e.getMessage());
            mTranslateStatus = FAILED;
        }
    }

    /**
     * Trigger a data read action from serial port
     */
    @Override
    public void readData() {
        if (null == mInput || null == mDataRecvCallback) return;
        new Thread(new Runnable() {
            @Override
            public synchronized void run() {
                byte[] buffer = new byte[1024];
                int len;
                try {
                    while (null != mInput) {
                        if (mInput.available() <= 0) {
                            try {
                                Thread.sleep(200);
                            } catch (InterruptedException e) {
                                //ignore
                            }
                            continue;
                        }
                        len = mInput.read(buffer);
                        //if (DEBUG) Log.d(TAG, "Read data: " + new String(buffer).substring(0, len));
                        mDataRecvCallback.onDataReceived(buffer, len);
                        break;
                    }
                } catch (IOException e) {
                    Log.e(TAG, "Read serial data failed! " + e.getMessage());
                }
            }
        }).start();
    }

    public void readDataNoWait() {
        if (null == mDataRecvCallback) return;
        byte[] buffer = new byte[1024];
        int len = 0;
        try {
            len = mInput.read(buffer);
        } catch (IOException e) {
            Log.e(TAG, "Read serial data failed! " + e.getMessage());
        }
        if (len > 0) {
            mDataRecvCallback.onDataReceived(buffer, len);
        }
    }

    public TranslateStatus getTranslateStatus() {
        return mTranslateStatus;
    }

    public SerialPort getSerialPort() {
        return mSerialPort;
    }
}
